Texture

Struct Texture 

pub struct Texture {
    pub label: String,
    pub texture: Texture,
    pub format: TextureFormat,
    pub view: TextureView,
    pub sampler: Sampler,
    pub size: (u32, u32),
    pub sample_count: u32,
    pub layer_count: u32,
    pub mip_level_count: u32,
}
Texture wrapper with a ready-to-use view and sampler.

Examples

Create a render target and clear it:

use wde_wgpu::{texture::{Texture, TextureFormat, TextureUsages}, instance::RenderInstanceData};

let color = Texture::new(
    instance,
    "color-target",
    (1280, 720),
    TextureFormat::Rgba8Unorm,
    TextureUsages::RENDER_ATTACHMENT | TextureUsages::COPY_SRC,
);

Upload raw pixel data (RGBA8):

use wde_wgpu::{texture::{Texture, TextureFormat, TextureUsages}, instance::RenderInstanceData};

let texture = Texture::new(
    instance,
    "albedo",
    (512, 512),
    TextureFormat::Rgba8Unorm,
    TextureUsages::TEXTURE_BINDING | TextureUsages::COPY_DST,
);
texture.copy_from_buffer(instance, TextureFormat::Rgba8Unorm, pixels);
// Or
texture.copy_from_buffer_layered(instance, TextureFormat::Rgba8Unorm, 0, pixels); // For texture arrays

Copy one GPU texture into another:


let dst = Texture::new(
    instance,
    "blit-target",
    (1024, 1024),
    TextureFormat::Rgba8Unorm,
    TextureUsages::COPY_DST | TextureUsages::TEXTURE_BINDING,
);
dst.copy_from_texture(instance, src, (1024, 1024));

Fields

label: Stringtexture: Textureformat: TextureFormatview: TextureViewsampler: Samplersize: (u32, u32)sample_count: u32layer_count: u32mip_level_count: u32

Implementations

Source

impl Texture

pub fn new( instance: &RenderInstanceData<'_>, label: &str, size: (u32, u32), format: TextureFormat, usage: TextureUsages, sample_count: u32, layer_count: u32, mip_level_count: u32, ) -> Self

Create a new texture.

Arguments
  • instance - Game instance.
  • label - Label of the texture. This is only for debugging purposes.
  • size - Size of the texture (width, height).
  • format - Format of the texture (e.g. Rgba8Unorm, Depth32Float, etc.).
  • usage - Usage of the texture (e.g. RENDER_ATTACHMENT, COPY_SRC, COPY_DST, etc.).
  • sample_count - Sample count of the texture (e.g. 1 for no MSAA, 4 for 4x MSAA, etc.).
  • layer_count - Number of layers in the texture array. Default is 1 (for non-array textures).
  • mip_level_count - Number of mip levels. Default is 1 (no mipmaps). If set to 0, it will be auto-calculated based on the texture size.
pub fn copy_from_buffer( &self, instance: &RenderInstanceData<'_>, texture_format: TextureFormat, buffer: &[u8], )

Copy buffer to texture. It is assumed that the buffer is the same size as the texture. It will be copied on the next queue submit. Note that the buffer must have the COPY_DST usage.

Arguments
  • instance - Game instance.
  • texture_format - The wgpu texture format.
  • buffer - Image buffer.
pub fn copy_from_buffer_layered( &self, instance: &RenderInstanceData<'_>, texture_format: TextureFormat, array_layer: u32, buffer: &[u8], )

Copy texture to buffer at a given array layer. It is assumed that the buffer is the same size as the texture. It will be copied on the next queue submit. Note that the buffer must have the COPY_DST usage.

Arguments
  • instance - Game instance.
  • texture_format - The wgpu texture format.
  • array_layer - The array layer to copy from (for texture arrays). Default is 0 for non-array textures.
  • buffer - Image buffer.
pub fn copy_from_texture( &self, instance: &RenderInstanceData<'_>, texture: &Texture, size: (u32, u32), )

Copy texture to texture. It is assumed that the texture is the same size as the source texture. Note that the input texture must have the COPY_SRC usage, and the output texture must have the COPY_DST usage.

Arguments
  • instance - Game instance.
  • texture - Texture to copy from.
  • size - Size of the texture.
pub fn copy_from_surface_texture( &self, instance: &RenderInstanceData<'_>, surface_texture: &SurfaceTexture, size: (u32, u32), )

Copy texture from a surface texture (e.g. swapchain frame). It is assumed that the texture is the same size as the source texture. Note that the input texture must have the COPY_SRC usage, and the output texture must have the COPY_DST usage.

Arguments
  • instance - Game instance.
  • surface_texture - Surface texture to copy from.
  • size - Size of the texture.
pub fn copy_from_texture_layered( &self, instance: &RenderInstanceData<'_>, texture: &Texture, array_layer: usize, size: (u32, u32), )

Copy texture to texture at a given array layer. It is assumed that the texture is the same size as the source texture. Note that the input texture must have the COPY_SRC usage, and the output texture must have the COPY_DST usage.

Arguments
  • instance - Game instance.
  • texture - Texture to copy from.
  • array_layer - The array layer to copy to (for texture arrays). Default is 0 for non-array textures.
  • size - Size of the texture.
pub fn generate_mipmaps(&self, instance: &RenderInstanceData<'_>)

Generate mipmaps for this texture. The texture must have been created with TEXTURE_BINDING | RENDER_ATTACHMENT | COPY_SRC usage. This method uses a simple blit approach to downsample each mip level from the previous one.

Arguments
  • instance - Game instance.