SurfaceTexture

Type Alias SurfaceTexture 

pub type SurfaceTexture = SurfaceTexture;
Surface texture.

Aliased Type

pub struct SurfaceTexture {
    pub texture: Texture,
    pub suboptimal: bool,
    /* private fields */
}

Fields

texture: Texture

Accessible view of the frame.

suboptimal: bool

true if the acquired buffer can still be used for rendering, but should be recreated for maximum performance.